/**
 * @brief   This example enables the listen-only mode and prints all received CAN frames to the serial interface.
 * @author  Alexander Entinger, MSc / LXRobotics GmbH
 */

/**************************************************************************************
 * INCLUDE
 **************************************************************************************/

#include <SPI.h>

#include <ArduinoMCP2515.h>

#undef max
#undef min
#include <algorithm>

/**************************************************************************************
 * FUNCTION DECLARATION
 **************************************************************************************/

void    mcp2515_spi_select           ();
void    mcp2515_spi_deselect         ();
uint8_t spi_transfer                 (uint8_t const);
void    mcp2515_onExternalEvent      ();
void    mcp2515_onReceiveBufferFull  (uint32_t const, uint8_t const *, uint8_t const);
void    mcp2515_onTransmitBufferEmpty(ArduinoMCP2515 *);

/**************************************************************************************
 * GLOBAL VARIABLES
 **************************************************************************************/

ArduinoMCP2515 mcp2515(mcp2515_spi_select,
                       mcp2515_spi_deselect,
                       spi_transfer,
                       mcp2515_onReceiveBufferFull,
                       mcp2515_onTransmitBufferEmpty);

/**************************************************************************************
 * SETUP/LOOP
 **************************************************************************************/

void setup()
{
  Serial.begin(9600);
  while(!Serial) { }

  /* Setup SPI access */
  SPI.begin();
  pinMode(MKRCAN_MCP2515_CS_PIN, OUTPUT);
  digitalWrite(MKRCAN_MCP2515_CS_PIN, HIGH);

  /* Attach interrupt handler to register MCP2515 signaled by taking INT low */
  pinMode(MKRCAN_MCP2515_INT_PIN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(MKRCAN_MCP2515_INT_PIN), mcp2515_onExternalEvent, FALLING);

  mcp2515.begin();
  mcp2515.setBitRate(CanBitRate::BR_250kBPS);
  mcp2515.setListenOnlyMode();
}

void loop()
{

}

/**************************************************************************************
 * FUNCTION DEFINITION
 **************************************************************************************/

void mcp2515_spi_select()
{
  digitalWrite(MKRCAN_MCP2515_CS_PIN, LOW);
}

void mcp2515_spi_deselect()
{
  digitalWrite(MKRCAN_MCP2515_CS_PIN, HIGH);
}

uint8_t spi_transfer(uint8_t const data)
{
  return SPI.transfer(data);
}

void mcp2515_onExternalEvent()
{
  mcp2515.onExternalEventHandler();
}

void mcp2515_onReceiveBufferFull(uint32_t const id, uint8_t const * data, uint8_t const len)
{
  Serial.print("ID");
  if(id & MCP2515::CAN_EFF_BITMASK) Serial.print("(EXT)");
  if(id & MCP2515::CAN_RTR_BITMASK) Serial.print("(RTR)");
  Serial.print(" ");
  Serial.print(id, HEX);

  Serial.print(" DATA[");
  Serial.print(len);
  Serial.print("] ");
  std::for_each(data,
                data+len,
                [](uint8_t const elem) {
                  Serial.print(elem, HEX);
                  Serial.print(" ");
                });
  Serial.println();
}

void mcp2515_onTransmitBufferEmpty(ArduinoMCP2515 * this_ptr)
{

}
